@@ -82,11 +82,17 @@ static xs_str *format_line(const char *line, xs_list **attach)
 /* formats a line */
 {
     xs_str *s = xs_str_new(NULL);
-    char *p, *v;
+    char *p;
+    const char *v;
 
     /* split by markup */
     xs *sm = xs_regex_split(line,
-        "(`[^`]+`|\\*\\*?[^\\*]+\\*?\\*|https?:/" "/[^[:space:]]+)");
+        "("
+            "`[^`]+`"                   "|"
+            "\\*\\*?[^\\*]+\\*?\\*"     "|"
+            "\\[[^]]+\\]\\([^\\)]+\\)"  "|"
+            "https?:/" "/[^[:space:]]+"
+        ")");
     int n = 0;
 
     p = sm;
@@ -134,6 +140,21 @@ static xs_str *format_line(const char *line, xs_list **attach)
                     s = xs_str_cat(s, s1);
                 }
             }
+            else
+            if (*v == '[') {
+                /* markdown-like links [label](url) */
+                xs *w    = xs_strip_chars_i(xs_dup(v), "[)");
+                xs *l    = xs_split_n(w, "](", 1);
+
+                if (xs_list_len(l) == 2) {
+                    xs *link = xs_fmt("<a href=\"%s\">%s</a>",
+                            xs_list_get(l, 1), xs_list_get(l, 0));
+
+                    s = xs_str_cat(s, link);
+                }
+                else
+                    s = xs_str_cat(s, v);
+            }
             else
                 s = xs_str_cat(s, v);
         }
@@ -155,7 +176,8 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
     int in_pre = 0;
     int in_blq = 0;
     xs *list;
-    char *p, *v;
+    char *p;
+    const char *v;
 
     /* work by lines */
     list = xs_split(content, "\n");
@@ -234,7 +256,7 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
         /* traditional emoticons */
         xs *d = emojis();
         int c = 0;
-        char *k, *v;
+        const char *k, *v;
 
         while (xs_dict_next(d, &k, &v, &c)) {
             const char *t = NULL;
@@ -280,7 +302,8 @@ xs_str *sanitize(const char *content)
     xs_str *s = xs_str_new(NULL);
     xs *sl;
     int n = 0;
-    char *p, *v;
+    char *p;
+    const char *v;
 
     sl = xs_regex_split(content, "</?[^>]+>");
 
